@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: arg.c,v 1.23 95/09/12 07:09:15 abe Exp $";
+static char *rcsid = "$Id: arg.c,v 1.25 95/09/19 10:46:41 abe Exp $";
 #endif
 
 
@@ -50,6 +50,7 @@ ctrl_dcache(c)
 	char *c;			/* control string */
 {
 	MALLOC_S len;
+	int rc = 0;
 	
 	if (c == NULL) {
 		(void) fprintf(stderr,
@@ -60,22 +61,39 @@ ctrl_dcache(c)
  * Decode argument function character.
  */
 	switch (*c) {
-
-#if	defined(HASDOPTPATH)
 	case 'b':
 	case 'B':
-		DCstate = 1;
+		if (Setuidroot
+
+#if	!defined(WILLDROPGID)
+		||  Myuid
+#endif	/* !defined(WILLDROPGID) */
+
+		)
+			rc = 1;
+		else
+			DCstate = 1;
 		break;
 	case 'r':
 	case 'R':
-		DCstate = 2;
+		if (Setuidroot && *(c+1))
+			rc = 1;
+		else
+			DCstate = 2;
 		break;
 	case 'u':
 	case 'U':
-		DCstate = 3;
-		break;
-#endif	/* defined(HASDOPTPATH) */
+		if (Setuidroot
 
+#if	!defined(WILLDROPGID)
+		||  Myuid
+#endif	/* !defined(WILLDROPGID) */
+
+		)
+			rc = 1;
+		else
+			DCstate = 3;
+		break;
 	case 'i':
 	case 'I':
 		if (*(c+1) == '\0') {
@@ -87,26 +105,25 @@ ctrl_dcache(c)
 		(void) fprintf(stderr, "%s: unknown -D option: %s\n", Pn, c);
 		return(1);
 	}
-
-#if	defined(HASDOPTPATH)
+	if (rc) {
+		(void) fprintf(stderr,
+			"%s: -D option restricted to root: %s\n", Pn, c);
+		return(1);
+	}
 /*
  * Skip to optional path name and save it.
  */
 	for (c++; *c && (*c == ' ' || *c == '\t'); c++)
 		;
 	if ((len = strlen(c))) {
-		if ((DCpath = (char *)malloc(len + 1)) == NULL) {
+		if ((DCpathArg = (char *)malloc(len + 1)) == NULL) {
 			(void) fprintf(stderr,
 				"%s: no space for -D path: %s\n", Pn, c);
 			exit(1);
 		}
-		(void) strcpy(DCpath, c);
+		(void) strcpy(DCpathArg, c);
 	}
 	return(0);
-#else	/* !defined(HASDOPTPATH) */
-	return(1);
-#endif	/* defined(HASDOPTPATH) */
-
 }
 #endif	/* defined(HASDCACHE) */
 
@@ -581,7 +598,7 @@ usage(xv, fh)
 	int xv;				/* exit value */
 	int fh;				/* ``-F ?'' status */
 {
-	char buf[48];
+	char buf[MAXPATHLEN+1], *cp;
 	int i;
 
 	if (Fhelp || xv) {
@@ -648,15 +665,20 @@ usage(xv, fh)
 # endif	/* defined(HASNCACHE) */
 	    (void) fprintf(stderr, "  %-37.37s", buf);
 # if	defined(HASDCACHE)
-	    (void) fprintf(stderr,
-#  if	defined(HASDOPTPATH)
-		"-D D   i|b|r|u[path] (%s_hostname)\n", HASDCACHE);
-#  else	/* !defined(HASDOPTPATH) */
-		"-D D   i (%s_hostname)\n", HASDCACHE);
-#  endif	/* defined(HASDOPTPATH) */
-# else	/* !defined(HASDCACHE) */
-	    (void) fprintf(stderr, "\n");
+	    if (Setuidroot)
+		cp = "i|r";
+
+#  if	!defined(WILLDROPGID)
+	    else if (Myuid)
+		cp = "i|r<path>";
+#  endif	/* !defined(WILLDROPGID) */
+
+	    else
+		cp = "i|b|r|u[path]";
+	    (void) fprintf(stderr, "-D D   %s\n", cp);
 # endif	/* defined(HASDCACHE) */
+# else	/* !defined(HASDCACHE) */
+	    putc('\n', stderr);
 #endif	/* defined(HASNCACHE) || defined(HASDCACHE) */
 
 	    (void) fprintf(stderr, "  %-37.37s",
@@ -723,18 +745,16 @@ usage(xv, fh)
 
 	    (void) fprintf(stderr,
 		"  names  select named files or files on named file systems\n");
-	    (void) fprintf(stderr, "The security mode is %s.\n",
+	    (void) fprintf(stderr, "%s can list all files.",
 
 #if	defined(HASSECURITY)
-		"enabled -- only the root user may list all open files"
+		"Only root"
 #else	/* !defined(HASSECURITY) */
-		"disabled -- anyone may list all open files"
+		"Anyone"
 #endif	/* defined(HASSECURITY) */
 
 	    );
-
-	    (void) fprintf(stderr,
-		"Warnings are %s for inaccessible device directories.\n",
+	    (void) fprintf(stderr, "  Inaccessible /dev warnings are %s.\n",
 
 #if	defined(WARNDEVACCESS)
 		"enabled"
@@ -743,6 +763,28 @@ usage(xv, fh)
 #endif	/* defined(WARNDEVACCESS) */
 
 	    );
+
+#if defined(HASDCACHE)
+# if	defined(HASENVDC) || defined(HASPERSDC) || defined(HASSYSDC)
+	    cp = NULL;
+#  if	defined(HASENVDC)
+	    if (dcpath(1) >= 0)
+		cp = DCpath[1];
+#  endif	/* defined(HASENVDC) */
+#  if	defined(HASSYSDC)
+	    if (!cp)
+		cp = HASSYSDC;
+#  endif	/* defined(HASSYSDC) */
+#  if	defined(HASPERSDC)
+	    if (!cp && dcpath(1) >= 0)
+		cp = DCpath[3];
+#  endif	/* defined(HASPERSDC) */
+	    if (cp)
+		(void) fprintf(stderr,
+		    "%s is the default device cache file read path.\n", cp);
+# endif    /* defined(HASENVDC) || defined(HASPERSDC) || defined(HASSYSDC) */
+#endif	/* defined(HASDCACHE) */
+
 	}
 	if (fh) {
 	    (void) fprintf(stderr, "%s:\tID    field description\n", Pn);
