@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.20 95/09/01 08:37:49 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.21 95/09/17 08:45:19 abe Exp $";
 #endif
 
 
@@ -625,7 +625,7 @@ bad_clone_sect:
 		if (!Fwarn)
 		    (void) fprintf(stderr,
 			"%s: bad clone section header in %s: %s",
-			Pn, DCpath, buf);
+			Pn, DCpath[DCpathX], buf);
 		return(1);
 	    }
 	    (void) crc(buf, strlen(buf), &DCcksum);
@@ -641,7 +641,8 @@ bad_clone_sect:
 		if (fgets(buf, sizeof(buf), DCfs) == NULL) {
 		    if (!Fwarn)
 			(void) fprintf(stderr,
-			    "%s: bad clone line in %s: %s", Pn, DCpath, buf);
+			    "%s: bad clone line in %s: %s",
+			    Pn, DCpath[DCpathX], buf);
 		    return(1);
 		}
 		(void) crc(buf, strlen(buf), &DCcksum);
@@ -749,7 +750,7 @@ bad_pseudo_sect:
 		if (!Fwarn)
 		    (void) fprintf(stderr,
 			"%s: bad pseudo section header in %s: %s",
-			Pn, DCpath, buf);
+			Pn, DCpath[DCpathX], buf);
 		return(1);
 	    }
 	    (void) crc(buf, strlen(buf), &DCcksum);
@@ -767,7 +768,8 @@ bad_pseudo_sect:
 bad_pseudo_line:
 		    if (!Fwarn)
 			(void) fprintf(stderr,
-			    "%s: bad pseudo line in %s: %s", Pn, DCpath, buf);
+			    "%s: bad pseudo line in %s: %s",
+			    Pn, DCpath[DCpathX], buf);
 		    return(1);
 		}
 		(void) crc(buf, strlen(buf), &DCcksum);
