@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.15 95/08/25 14:03:48 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.18 95/09/20 20:13:34 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -60,6 +60,8 @@ static struct proc *P = NULL;		/* local proc structure table */
 #if	defined(solaris)
 static int *Pid = NULL;			/* PIDs for P[] entries */
 static int *Pgrp = NULL;		/* process group IDs for P[] entries */
+static gid_t Savedgid;			/* saved (effective) GID */
+static int Switchgid = 0;		/* must switch GIDs for kvm_open() */
 #endif	/* defined(solaris) */
 
 
@@ -370,19 +372,58 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
-	if ((Memory != NULL && !is_readable(Memory))
-	||  (Nmlst != NULL && !is_readable(Nmlst)))
+
+/*
+ * Get kernel symbols.
+ */
+	if (Nmlst && !is_readable(Nmlst, 1))
 		exit(1);
 	if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
 		(void) fprintf(stderr, "%s: can't read namelist from %s\n",
 			Pn, Nmlst ? Nmlst : N_UNIX);
 		exit(1);
 	}
-	if (Nl[X_NPROC].n_value) {
-		open_kvm();
-		if (kread((KA_T)Nl[X_NPROC].n_value, (char *)&Knp, sizeof(Knp))
-		||  Knp < 1)
-			Knp = PROCDFLT;
+
+#if	defined(WILLDROPGID)
+
+# if	defined(solaris)
+/*
+ * If Solaris kernel memory is coming from KMEM, and the process is willing
+ * to surrender GID permission, set up for GID switching after the first
+ * call to open_kvm().
+ */
+	if (!Memory) {
+		Savedgid = getegid();
+		if (Setgid)
+			Switchgid = 1;
+	}
+# endif	/* defined(solaris) */
+
+/*
+ * If kernel memory isn't coming from KMEM, drop setgid permission
+ * before attempting to open the (Memory) file.
+ */
+	if (Memory)
+		(void) dropgid();
+#else	/* !defined(WILLDROPGID) */
+/*
+ * See if the non-KMEM memory file is readable.
+ */
+	if (Memory && !is_readable(Memory, 1))
+		exit(1);
+#endif	/* defined(WILLDROPGID) */
+
+/*
+ * Open access to kernel memory.
+ */
+	open_kvm();
+/*
+ * Get a proc structure count estimate.
+ */
+	if (Nl[X_NPROC].n_value
+	&&  kread((KA_T)Nl[X_NPROC].n_value, (char *)&Knp, sizeof(Knp))
+	||  Knp < 1)
+		Knp = PROCDFLT;
 
 #if	defined(solaris)
 # if	solaris>=20500
@@ -413,12 +454,11 @@ get_kernel_access()
  *
  * Since the SunOS 4.1.x process table is sequential, we need not worry
  * about a stale or incorrect list head pointer.  Thus we don't have to call
- * close_kvm().
+ * close_kvm().  Instead, we call kvm_setproc() each time we read it.
  */
-		close_kvm();
+	close_kvm();
 #endif	/* defined(solaris) */
 
-	}
 }
 
 
@@ -505,6 +545,23 @@ open_kvm()
 {
 	if (Kd != NULL)
 		return;
+
+#if	defined(solaris) && defined(WILLDROPGID)
+/*
+ * If this Solaris process began with setgid permission and its been
+ * surrendered, regain it.
+ */
+	if (Switchgid == 2 && !Setgid) {
+		if (setgid(Savedgid) != 0) {
+			(void) fprintf(stderr,
+				"%s: can't set effective GID to %d: %s\n",
+				Pn, Savedgid, strerror(errno));
+			exit(1);
+		}
+		Setgid = 1;
+	}
+#endif	/* defined(solaris) && defined(WILLDROPGID) */
+
 	if ((Kd = kvm_open(Nmlst, Memory, NULL, O_RDONLY, NULL)) == NULL) {
 		(void) fprintf(stderr,
 			"%s: kvm_open (namelist=%s, core=%s): %s\n",
@@ -514,6 +571,24 @@ open_kvm()
 			strerror(errno));
 		exit(1);
 	}
+
+#if	defined(WILLDROPGID)
+/*
+ * If this process has setgid permission, and is willing to surrender it,
+ * do so.
+ */
+	(void) dropgid();
+
+# if	defined(solaris)
+/*
+ * If this Solaris process must switch GIDs, enable switching after the
+ * first call to this function.
+ */
+	if (Switchgid == 1)
+		Switchgid = 2;
+# endif	/* defined(solaris) */
+#endif	/* define(WILLDROPGID) */
+
 }
 
 
@@ -794,7 +869,11 @@ read_proc()
 	 */
 	    if (Np >= PROCMIN)
 		break;
+
+#if	defined(solaris)
 	    close_kvm();
+#endif	/* defined(solaris) */
+
 	}
 /*
  * Quit if no proc structures were stored in the local table.
