@@ -32,20 +32,31 @@
 
 /*
  * HASDCACHE is defined for those dialects that support a device cache
- * file.  HASDCACHE is the second component of its path.  The initial
- * component of the path will be the lsof user's home directory.  If
- * that's unusable, no device cache file will be created.  The last
- * component of the path will be the host name, separated from HASDCACHE
- * with an underscore.  If no host name is available, "noname" will be
- * used.
+ * file.
  *
- * HASDOPTPATH is defined for those dialects that allow a path name to
- * be specified with the -Db, -Dr, and -Du options.  Setuid(root)
- * dialects should not define HASDCPATH.
+ * HASENVDC defined the name of an environment variable that contains the
+ * device cache file path.  The HASENVDC environment variable is ignored when
+ * the lsof process is setuid(root) or its real UID is 0.
+ *
+ * HASPERSDC defines the format for the last component of a personal device
+ * cache file path.  The first will be the home directory of the real UID that
+ * executes lsof.
+ *
+ * HASPERSDCPATH defines the environment variable whose value is the middle
+ * component of the personal device cache file path.  The middle component
+ * follows the home directory and precedes the results of applying HASPERSDC.
+ * The HASPERSDCPATH environment variable is ignored when the lsof process is
+ * setuid(root) or its real UID is 0.
+ *
+ * HASSYSDC defines a public device cache file path.  When it's defined, it's
+ * used as the path from which to read the device cache.
  */
 
-#define	HASDCACHE	".lsof"
-#define	HASDOPTPATH	1
+#define	HASDCACHE	1
+#define	HASENVDC	"LSOFDEVCACHE"
+#define	HASPERSDC	".lsof_%L"
+#define	HASPERSDCPATH	"LSOFPERSDCPATH"
+/* #define	HASSYSDC	"/var/adm/lsof.dc" */
 
 
 /*
@@ -267,6 +278,15 @@
 #define	WARNDEVACCESS	1
 
 
+/*
+ * WILLDROPGID is defined for those dialects whose lsof executable runs
+ * setgid(not_real_GID) and whose setgid power can be relinquished after
+ * the dialect's initialize() function has been executed.
+ */
+
+#define	WILLDROPGID	1
+
+
 /*
  * zeromem is a macro that uses bzero or memset.
  */
