@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.17 95/08/22 13:03:51 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.19 95/09/19 10:45:55 abe Exp $";
 #endif
 
 
@@ -80,11 +80,10 @@ main(argc, argv)
  * Common initialization.
  */
 	Mypid = getpid();
-
-#if	defined(HASSECURITY)
-	Myuid = (uid_t)getuid();
-#endif
-
+	if ((Mygid = (gid_t)getgid()) != getegid())
+		Setgid = 1;
+	if ((Myuid = (uid_t)getuid()) && !geteuid())
+		Setuidroot = 1;
 	if ((Namech = (char *)malloc(MAXPATHLEN)) == NULL) {
 		(void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
 		exit(1);
@@ -398,6 +397,14 @@ main(argc, argv)
  */
 	initialize();
 
+#if	defined(WILLDROPGID)
+/*
+ * If this process isn't setuid(root), but it is setgid(not_real_gid),
+ * relinquish the setgid power.  (If it hasn't already been done.)
+ */
+	(void) dropgid();
+#endif	/* defined(WILLDROPGID) */
+
 #if	defined(HASDCACHE)
 /*
  * If there is a device cache, prepare the device table.
